/******************************************/
/*  Bowed Bar model                       */
/*  by Georg Essl, 1999                   */
/******************************************/

#include "BowedBar.h"
#include "SKINI11.msg"
#include "Noise.h"

// #define NR_MODES 4
int NR_MODES=4;

BowedBar :: BowedBar(MY_FLOAT lowestFreq)
{
  long i;
  
  pluck_ = 1;
  
  modes[0] = 1.0;
  modes[1] = 2.756;
  modes[2] = 5.404;
  modes[3] = 8.933;
  for (i=0;i<4;i++)     {
        gains[i] = pow(0.9,(double) i);
  }
  
  bowTabl = new BowTabl;
  adsr = new ADSR;
//  #ifndef HARD_BP
//   bandpass_ = new TwoPole[NR_MODES];
//    bandpass_ = new WGFilter[NR_MODES];
  bandpass_ = new BiQuad[NR_MODES];
//  #endif
  
  bowTabl->setSlope((MY_FLOAT) 3.0);
  adsr->setAllTimes((MY_FLOAT) 0.02,(MY_FLOAT) 0.005,(MY_FLOAT) 0.9,(MY_FLOAT) 0.01);
    
  freq = SRATE / 100;
  length = 100;
  
  bowPos = 0;
  lastBowPos = 0;

  for(i = 0; i<NR_MODES; i++)
  {
    delay[i].setDelay((int)(length/modes[i]));
    delay[i].clear();
    
//  #ifndef HARD_BP
    bandpass_[i].clear();
//  #else
    Zs[i][1] = 0.0;
    Zs[i][2] = 0.0;
    filtOut[i] = 0.0;
    filtIn[i] = 0.0;
//  #endif    
  }
  
//  printf("freq: %f len: %d\n", freq, length);
  
//   hairnoise = 0.0;
  
  R = 0.97;
//   GAIN = 1.0;
  GAIN = 0.999;
//   GAIN = 0.0;
  
  slope = 3.0;
  
  tuneBandPasses();
}

BowedBar :: ~BowedBar()
{
  delete bowTabl;
  delete adsr;
  
}

void BowedBar :: clear()
{
  long i;
  
  for(i = 0; i<NR_MODES; i++)
  {
    delay[i].clear();
    
//  #ifndef HARD_BP
    bandpass_[i].clear();
//  #else
    Zs[i][1] = 0.0;
    Zs[i][2] = 0.0;
    filtOut[i] = 0.0;
    filtIn[i] = 0.0;
//  #endif
  }
}

void BowedBar :: setFreq(MY_FLOAT frequency)
{
  int i;
  freq = frequency;

  if(freq > 1568) freq = 1568.0;

  length = (int) (SRATE/freq);
  
  NR_MODES = 4;
  for(i = 0; i<NR_MODES; i++)
  {
    if((int)(length/modes[i]) > 4)
       delay[i].setDelay((int)(length/modes[i]));
    else        {
      NR_MODES = i;
      break;
    }
/*   FIX THIS BETTER!!!!! */
    delay[i].clear();
    
//  #ifndef HARD_BP
    bandpass_[i].clear();
//  #else
    Zs[i][1] = 0.0;
    Zs[i][2] = 0.0;
    filtOut[i] = 0.0;
    filtIn[i] = 0.0;
//  #endif
  }
//  printf("freq: %f len: %d\n", freq, length);
  
  tuneBandPasses();
}

void BowedBar :: setStrikePosition(MY_FLOAT position)
{
    MY_FLOAT temp2;
    temp2 = position * PI;
//    strikePosition = position;             /*  Hack only first three modes */
    gains[0] = fabs(sin(temp2 / 2) * pow(0.9,0));                               
        
    gains[1] = fabs(sin(temp2) * pow(0.9,1));
    gains[2] = fabs(sin(temp2 * 3 / 2) * pow(0.9,2));
    gains[3] = fabs(sin(temp2 * 2) * pow(0.9,3));
//    printf("%f %f %f %f\n",gains[0],gains[1],gains[2],gains[3]);
}

void BowedBar :: tuneBandPasses()
{
  long i;
  
  for(i=0; i<NR_MODES; i++)
  {
    R = 1 - 6.28318530718 * freq * modes[i] / SRATE / 2.0;
//  #ifndef HARD_BP
    bandpass_[i].setFreqAndReson(freq * modes[i], R);
    bandpass_[i].setEqualGainZeroes();
    bandpass_[i].setGain((1.0-R*R)/2.0);
//     bandpass_[i].setGain2((1.0 - (R*R))/2.0);
//  #else
    filtGain[i] = (1.0 - (R*R))/2.0;
    coeffs[i][1] = - R * R;
    coeffs[i][0] = 2.0 * R * cos(6.28318530718 * freq * modes[i] / SRATE);
//  #endif
    
//       for (j=0;j<length/modes[i];j++) delay[i][j] = (rand()) - 16384;
//     for (j=0;j<length/modes[i];j++) delay[i][j] = 0; //(rand()) - 16384;
    delay[i].clear(); //(rand()) - 16384;
  }
}

void BowedBar :: startBowing(MY_FLOAT amplitude, MY_FLOAT rate)
{
    adsr->setRate(rate);
    adsr->keyOn();
    maxVelocity = (MY_FLOAT) 0.03 + ((MY_FLOAT) 0.2 * amplitude); 
    maxVelocity = (MY_FLOAT) 0.03 + ((MY_FLOAT) 0.5 * amplitude); 
//     maxVelocity = 0;
}

void BowedBar :: stopBowing(MY_FLOAT rate)
{
    adsr->setRate(rate);
    adsr->keyOff();
}

void BowedBar :: pluck(MY_FLOAT amplitude)
{
  long i,j;
  int pluckLen;
  MY_FLOAT temp;
  Noise noise;
  
    pluckLen = (int) (length/modes[NR_MODES-1]);
    for (j=1;j<pluckLen/2;j++)  {
        temp = amplitude*2.0*noise.tick();
            for(i=0; i<NR_MODES; i++)
               delay[i].tick(temp*j/pluckLen*gains[i]);
    }
    for (j=pluckLen/2;j>0;j--)  {
        temp = amplitude*2.0*noise.tick();
            for(i=0; i<NR_MODES; i++)
               delay[i].tick(temp*j/pluckLen*gains[i]);
    }
}

void BowedBar :: noteOn(MY_FLOAT freq, MY_FLOAT amp)
{
  if(!pluck_)
  {
  for(int i=0; i<NR_MODES ; i++)
    bandpass_[i].clear();
  
    this->startBowing(amp,amp * (MY_FLOAT) 0.001);
    this->setFreq(freq);
    pluck_ = 0;
#if defined(_debug_)        
    printf("BowedBar : NoteOn: Freq=%lf Amp=%lf\n",freq,amp);
#endif    
  }
  else
  {
          for(int i=0; i<NR_MODES ; i++)
        bandpass_[i].clear();
          this->setFreq(freq);
          this->pluck(amp);
          pluck_ = 1;
  }
}

void BowedBar :: noteOff(MY_FLOAT amp)
{
  if(!pluck_)
  {
  this->stopBowing(((MY_FLOAT) 1.0 - amp) * (MY_FLOAT) 0.005);
//  this->clear();
  }
//    this->pluck(amp);
//    pluck_ = 1;
#if defined(_debug_)        
    printf("BowedBar : NoteOff: Amp=%lf\n",amp);
#endif    
}

// void BowedBar :: setVibrato(MY_FLOAT amount)
// {
//     vibrGain = amount;
// }

MY_FLOAT BowedBar :: tick()
{
  long k;
//   MY_FLOAT velinput;
  MY_FLOAT input;
  MY_FLOAT data;
  MY_FLOAT temp=0.0;
//   Noise noise;
  
  data = 0.0;
      
  input = 0.0;
  if(integration_const_ == 0.0)
    velinput = 0.0;
  else
    velinput = integration_const_ * velinput;
//   temp = velinput;
//  velinput = (1.0+hairnoise*noise.tick())/2.0;
  for(k=0; k<NR_MODES; k++)
  {
    velinput += GAIN * delay[k].lastOut();
  }
//  temp = velinput;
//  for(k=0; k<NR_MODES; k++)
//  {
//    temp -= GAIN*delay[k].lastOut();
//  }
  
  if(trackVel)  {
    bowvel *= 0.9995;
    bowvel += bowTarg;
    bowTarg *= 0.995;
  }
  else  {
    bowvel = adsr->tick()*maxVelocity;
  }

  if(pluck_)
  {
    input = 0.0;
  }
  else
  {
    input = bowvel - velinput;
  
    input = input * bowTabl->lookup(input);
  
    input = input/(MY_FLOAT)NR_MODES;
  }
  
  for(k=0; k<NR_MODES; k++)
  {
//  #ifndef HARD_BP
    bandpass_[k].tick(input*gains[k] + GAIN * delay[k].lastOut());
//      delay[k].tick(bandpass_[k].lastOut());
//  #else
/*    filtIn[k] = input + GAIN * delay[k].lastOut();
    
    filtIn[k] += Zs[k][0] * coeffs[k][0];
    filtIn[k] += Zs[k][1] * coeffs[k][1];
    filtOut[k] = filtGain[k] * (filtIn[k] - Zs[k][1]);

    Zs[k][1] = Zs[k][0];
    Zs[k][0] = filtIn[k];
        
    if(fabs(bandpass_[k].lastOut()-filtOut[k])>0.00001) printf("%f %f\n", bandpa
ss_[k].lastOut(), filtOut[k]);
    */
//      delay[k].tick(filtOut[k]);
    delay[k].tick(bandpass_[k].lastOut());
//  #endif
    data += bandpass_[k].lastOut();        //    delay[k].lastOut();///0.4;
  }
//   print("data: %d\n", data);
  
  lastOutput = data * 4.0;
  return lastOutput;
}

void BowedBar :: controlChange(int number, MY_FLOAT value)
{
#if defined(_debug_)        
    printf("BowedBar : ControlChange: Number=%i Value=%f\n",number,value);
#endif    
    if (number == __SK_BowPressure_)
    {
      bowTabl->setSlope((MY_FLOAT) 10.0 - ((MY_FLOAT) 9.0 * value * NORM_7));
      slope = (MY_FLOAT) 10.0 - ((MY_FLOAT) 9.0 * value * NORM_7);
//!      printf("slope = %f\n", slope);
    }
    else if (number == __SK_BowPosition_)
    {
        this->setStrikePosition(value*NORM_7);
//      NR_MODES = (int)(4.99*value*NORM_7);
//!      printf("NR_MODES = %d\n", NR_MODES);
//       bowvel = value * NORM_7;
//       printf("bowvel = %f\n",bowvel);
    }
    else if (number == __SK_Balance_)   {
      bowPos = value * NORM_7;
      bowTarg += 0.02 * (bowPos - lastBowPos);
      lastBowPos = bowPos;
      adsr->setTarget(bowPos);
//      printf("BReath -> %f\n",bowTarg);
    }
    else if (number == __SK_AfterTouch_Cont_)
    {
      bowPos = value * NORM_7;
      bowTarg += 0.02 * (bowPos - lastBowPos);
      lastBowPos = bowPos;
      adsr->setTarget(bowPos);
//!      printf("adsr = %f\n", value * NORM_7);
//       GAIN = value * NORM_7;
//       printf("GAIN: %f\n", GAIN);
    }      
    else if (number == __SK_ModWheel_)
    {
      GAIN = 0.809 + (0.2 * (value * NORM_7));
//!      printf("GAIN: %f\n", GAIN);
    }
    else if(number == __SK_ModFrequency_)
    {
//       hairnoise = value * NORM_7;
//       printf("Hairnoise: %f\n", hairnoise);
      integration_const_ = value * NORM_7;
//!      printf("Integrator-Slope: %f\n", integration_const_);
      
    }
    else if(number == __SK_Sustain_)    {
             if(value < 65)
                pluck_ = 1;
             else
                pluck_ = 0;
             printf("pluck_ = %i\n", pluck_);
    }
    else if(number == __SK_Portamento_) {
             if(value < 65)
                trackVel = 0;
             else
                trackVel = 1;
             printf("trackVel = %i\n", trackVel);
    }
    else    {        
        printf("BowedBar : Undefined Control Number!!\n");
    }    
}

