// gaitlab.cpp
//
// An STK program for synthesis of walking

#include <windows.h>
#include <windowsx.h>
#include <stdarg.h>


#include "SKINI11.h"
#include "SKINI11.msg"
#include "miditabl.h"

int numStrings = 0;
int notDone = 1;
char **inputString;

// Miscellaneous command-line parsing and instrument allocation
// functions are defined in utilites.cpp.
#include "utilities.h"

// The input command pipe and socket threads are defined in threads.cpp.
#include "threads.h"

#define TIMEGRAIN 0.01
#define PHASES 8

  char hostName[256];
  int theSocket = 0;
  struct sockaddr_in saServer;
  char s[128] = {"//////******  CONNECTION FROM GAITLAB  *******\n"};

void doSocketOut(void)	{
	/**********************    SOCKET TO SYNTH SERVER  ***********/

  // setup the client socket
    WSADATA wsaData;
    WORD wVersionRequested = MAKEWORD(1,1);

    WSAStartup(wVersionRequested, &wsaData);
	if (wsaData.wVersion != wVersionRequested) {	
		fprintf(stderr,"\n Wrong Windoze socket library version!\n");
		exit(0);
	}

		// Create the client-side socket

	theSocket = socket(AF_INET,	SOCK_STREAM, IPPROTO_TCP);
	if (theSocket < 0) {
		fprintf(stderr,"Couldn't create socket ... aborting!\n");
		exit(0);
	}

    struct hostent *hostp;

    if ((hostp = gethostbyname(hostName)) == 0) {
      fprintf(stderr,"%s: unknown host ... aborting!\n", hostName);
      exit(0);
    }

		// Fill in the address structure
    saServer.sin_family = AF_INET;
    memcpy((void *)&saServer.sin_addr, hostp->h_addr, hostp->h_length);
		saServer.sin_port = htons(2001);	// Port number


		// Connect to the server
	if(connect(theSocket, (struct sockaddr *)&saServer, sizeof(saServer)) < 0) {
		fprintf(stderr,"Socket connect failed ... aborting!\n");
		closesocket(theSocket);
		WSACleanup();
		exit(0);
	}

    if (send(theSocket, s,	strlen(s), 0) < 0) {
		fprintf(stderr,"Socket connection failed ... aborting.\n");
		closesocket(theSocket);
		WSACleanup();
		exit(0);
	}		
}
		
int main(int argc,char *argv[])
{
  long i;
  int rtInput;
  int type, phase = 0;
  int outOne = 0;
  int walking = 1;
  int byteTwoInt;
  FILE *fileIn = stdin;
  double gaitTimes[PHASES+1] = {0.0, 0.1, 0.2, 0.3, 1.0, 1.1, 1.2, 1.3, 2.0};
  double gaitGains[PHASES+1] = {0.0, 0.9, 0.3, 0.0, 0.0, 0.9, 0.3, 0.0, 0.0};
  double temp, timeScale = 1.0;
  double envGain = 0.0, lenvGain = 0.0;
  double deltaGain = 0;
  double envTime = 0.0;  
  
  SKINI11 *score = new SKINI11();

  rtInput = 2;

  //  If using realtime input, start the input thread.
  if (rtInput) {
//#if defined(__STK_REALTIME_)
//    if (rtInput == 2) 
		startSocketThread();
//    else 
//		startPipeThread();
//    instrument->noteOn(200.0,0.1);
//#endif
  }
  else { // Malloc inputString as 1 x STRING_LEN array
    inputString = (char **) malloc(sizeof(char *));
    inputString[0] = (char *) malloc(STRING_LEN * sizeof(char));
  }

  doSocketOut();
  
  // The runtime loop begins here:
  notDone = 1;
  while(notDone || numStrings)  {
      Sleep(TIMEGRAIN / 0.001);
	  if (walking)	{
		temp = (gaitTimes[phase+1] - envTime) / TIMEGRAIN;
		deltaGain = (gaitGains[phase+1] - envGain) / temp;
		envGain += deltaGain * timeScale;
		envTime += TIMEGRAIN * timeScale;
		if (envTime >= gaitTimes[phase+1])	{
		  phase += 1;
		  envGain = gaitGains[phase];
		  if (phase >= PHASES)	{
			  phase = 0;
		      envTime = 0.0;
		  }
		}
		if (fabs(lenvGain - envGain) > 0.001)	{
		  lenvGain = envGain;
		  sprintf(s,"AfterTouch 0.0 0 %f\n",envGain * 127);
	      send(theSocket, s,	strlen(s), 0);
		}
	  }

    if (numStrings) {
//	  printf(inputString[outOne]);
	  if (inputString[outOne][1]=='x') 
		  notDone = 0;
	  if (notDone)	{
		  score->parseThis(inputString[outOne]);
		  type = score->getType();
		if (type > 0) {
           switch(type) {

           case __SK_NoteOn_:
			walking = 1;
			// envTime = 0;
			strcpy(s,inputString[outOne]);
			send(theSocket, s,	strlen(s), 0);

          break;

           case __SK_NoteOff_:
			walking = 0;
		  break;

           case __SK_ControlChange_:
			byteTwoInt = score->getByteTwoInt();
			temp = score->getByteThree() / 128.0;
			if      (byteTwoInt == 701) gaitTimes[0] = 2*temp;
			else if (byteTwoInt == 702) gaitTimes[1] = 2*temp;
			else if (byteTwoInt == 703) gaitTimes[2] = 2*temp;
			else if (byteTwoInt == 704) gaitTimes[3] = 2*temp;
			else if (byteTwoInt == 705) gaitTimes[4] = 2*temp;
			else if (byteTwoInt == 706) gaitTimes[5] = 2*temp;
			else if (byteTwoInt == 707) gaitTimes[6] = 2*temp;
			else if (byteTwoInt == 708) gaitTimes[7] = 2*temp;
			
			else if (byteTwoInt == 801) {
				gaitGains[0] = temp;
				gaitGains[8] = temp;
			}
			else if (byteTwoInt == 802) gaitGains[1] = temp;
			else if (byteTwoInt == 803) gaitGains[2] = temp;
			else if (byteTwoInt == 804) gaitGains[3] = temp;
			else if (byteTwoInt == 805) gaitGains[4] = temp;
			else if (byteTwoInt == 806) gaitGains[5] = temp;
			else if (byteTwoInt == 807) gaitGains[6] = temp;
			else if (byteTwoInt == 808) gaitGains[7] = temp;

			else if (byteTwoInt == 601) {
				timeScale = score->getByteThree() / 60.0;
//				printf("timeScale is: %f\n",timeScale);
			}
			
			else {
				strcpy(s,inputString[outOne]);
				send(theSocket, s,	strlen(s), 0);
			}
				
//			for (i=0;i<8;i++) printf("%.3f ",gaitTimes[i]);
//			printf("\n");
			break;
		}
		if (rtInput) {
            outOne += 1;
            if (outOne == MAX_IN_STRINGS) outOne = 0;
		}
		numStrings--;
		}
	  }
	}

  }
  if (!rtInput) { // Free the inputString for non-realtime operation.
    free(inputString[0]);
    free(inputString);
  }

  Sleep((DWORD) 200);
  delete score;
  Sleep((DWORD) 200);

strcpy(s, "ExitProgram\n");
send(theSocket, s,	strlen(s), 0);
Sleep((DWORD) 20);

	closesocket(theSocket);
    WSACleanup();

        printf("Bill's Tired Now.  Buh Bye!\n");
  return 0;
}

