/*******************************************/
/*  ModalBar SubClass of Modal4 Instrument */
/*  by Perry R. Cook, 1999                 */ 
/*                                         */
/*   Controls:    CONTROL1 = stickHardness */
/*                CONTROL2 = strikePosition*/
/*		  CONTROL3 = Mode Presets  */
/*		  MOD_WHEEL= ??????        */
/*******************************************/

#include "ModalBar.h"
#include "SKINI11.msg"

ModalBar :: ModalBar() : Modal4()
{
    wave = new RawWvIn("rawwaves/expnoise.raw","oneshot");
    wave->normalize();
    wave->setRate((MY_FLOAT) 0.5);                      /*  normal stick  */
//    inputFilt.setPole(0.8);
    this->setRatioAndReson(0, (MY_FLOAT) 1.00,(MY_FLOAT) 0.9996);  /*  Set all       132.0  */
    this->setRatioAndReson(1, (MY_FLOAT) 3.99,(MY_FLOAT) 0.9994);  /*  of our        523.0  */
    this->setRatioAndReson(2,(MY_FLOAT) 10.65,(MY_FLOAT) 0.9994);  /*  default       1405.0 */
    this->setRatioAndReson(3,-(MY_FLOAT) 2443.0,(MY_FLOAT) 0.999);  /*  resonances    2443.0 */
    gains[0] = 0.04;
    gains[1] = 0.01;
    gains[2] = 0.01;
    gains[3] = 0.008;
	this->setFiltGain(0,gains[0]);               /*  and        */
    this->setFiltGain(1,gains[1]);               /*  gains      */
    this->setFiltGain(2,gains[2]);               /*  for each   */
    this->setFiltGain(3,gains[3]);              /*  resonance  */
    directGain = (MY_FLOAT) 0.1;
}  

ModalBar :: ~ModalBar()
{
    delete wave;
}  

void ModalBar :: setStickHardness(MY_FLOAT hardness)
{ 
	MY_FLOAT temp;
	stickHardness = hardness;
    temp = (MY_FLOAT) ((MY_FLOAT)  pow(8.0,stickHardness) / 6.0);
	temp = temp * temp;
//	printf("%f\n",temp);
	wave->setRate(temp);

//    inputFilt.setPole(1.998 * temp); // 0.999 to -0.999
//	masterGain = pow(1.0 + (2.0 * fabs(temp)),3);  //  
    masterGain = (MY_FLOAT) 0.1 + (MY_FLOAT) 1.8*stickHardness;
}

void ModalBar :: setStrikePosition(MY_FLOAT position)
{
    MY_FLOAT temp,temp2;
    temp2 = position * PI;
    strikePosition = position;             /*  Hack only first three modes */
    temp = (MY_FLOAT)  sin(temp2);                                       
    this->setFiltGain(0, gains[0] * temp);      /*  1st mode function of pos.   */
    temp = (MY_FLOAT)  sin(0.05 + (resons[1] * temp2));
    this->setFiltGain(1,gains[1] * temp);     /*  2nd mode function of pos.   */
    temp = (MY_FLOAT)  sin(-0.05 + (resons[2] * temp2));
    this->setFiltGain(2,gains[2] * temp);      /*  3rd mode function of pos.   */
}

void ModalBar :: setModalPreset(int which)
{
    int i,temp;
    MY_FLOAT presets[8][4][4] = { 
			{{1.0, 3.99, 10.65, -2443},		// Marimba
				{0.9996, 0.9994, 0.9994, 0.999},
				{0.04, 0.01, 0.01, 0.008},
				{0.429688, 0.445312, 0.093750}},
			{{1.0, 4.08, 6.669, -3725.0},		// Agogo 
				{0.999, 0.999, 0.999, 0.999},	
				{0.06, 0.05, 0.03, 0.02},
				{0.609375,0.359375,0.140625}},
			{{1.0, 2.777, 7.378, 15.377},		// Wood1
				{0.996, 0.994, 0.994, 0.99},	
				{0.04, 0.01, 0.01, 0.008},
				{0.460938,0.375000,0.046875}},
			{{1.0, 2.777, 7.378, 15.377},		// Reso
				{0.99996, 0.99994, 0.99994, 0.9999},	
				{0.02, 0.005, 0.005, 0.004},
				{0.453125,0.250000,0.101562}},
			{{1.0, 1.777, 2.378, 3.377},		// Wood2
				{0.996, 0.994, 0.994, 0.99},	
				{0.04, 0.01, 0.01, 0.008},
				{0.312500,0.445312,0.109375}},
			{{1.0, 1.004, 1.013, 2.377},		// Beats
				{0.9999, 0.9999, 0.9999, 0.999},	
				{0.02, 0.005, 0.005, 0.004},
				{0.398438,0.296875,0.070312}},
			{{1.0, 4.0, -1320.0, -3960.0},		// 2Fix
				{0.9996, 0.999, 0.9994, 0.999},	
				{0.04, 0.01, 0.01, 0.008},
				{0.453125,0.453125,0.070312}},
			{{1.0, 1.217, 1.475, 1.729},		// Clump
				{0.999, 0.999, 0.999, 0.999},	
				{0.03, 0.03, 0.03, 0.03 },
				{0.390625,0.570312,0.078125}},
			     };

    temp = ((which / 16) % 8);
    for (i=0;i<4;i++)	{
    	this->setRatioAndReson(i, presets[temp][0][i],
		presets[temp][1][i]);
		gains[i] = presets[temp][2][i];
    	this->setFiltGain(i,gains[i]);
    }
    this->setStickHardness(presets[temp][3][0]);
    this->setStrikePosition(presets[temp][3][1]);
    directGain = presets[temp][3][2];
}

void ModalBar :: setModalParam(int whichFilt,MY_FLOAT ratio,
									MY_FLOAT reson,MY_FLOAT gain)
{
	gains[whichFilt] = gain;
    this->setRatioAndReson(whichFilt,ratio,reson);
    this->setFiltGain(whichFilt,gain);	
}

void ModalBar :: controlChange(int number, MY_FLOAT value)
{
#if defined(_debug_)        
    printf("ModalBar : ControlChange: Number=%i Value=%f\n",number,value);
#endif    
    if (number == __SK_StickHardness_)
        this->setStickHardness(value * NORM_7);
    else if (number == __SK_StrikePosition_)
        this->setStrikePosition(value * NORM_7);
    else if (number == __SK_ProphesyRibbon_)
		this->setModalPreset((int) value);
    else if (number == __SK_ModWheel_)
	directGain = value * NORM_7;
    else if (number == __SK_AfterTouch_Cont_)	{
    	envelope->setTarget(value * NORM_7);
 //       this->strike(value * NORM_7);
    }
    else if (number == 1024)	{	//  HACKED Poop message
        //  printf("StickHard=%f   StrikePos=%f   directGain=%f\n",
		printf("ModalFreq 0.0 0 %f %f\n",baseFreq,64.0);
		printf("ModalInput 0.0 0 %f %f %f %f\n",
				stickHardness, strikePosition, directGain);
		printf("ModalParam 0.0 0 0 %f %f %f\n",
					ratios[0],resons[0],gains[0]);
		printf("ModalParam 0.0 0 1 %f %f %f\n",
					ratios[1],resons[1],gains[1]);
		printf("ModalParam 0.0 0 2 %f %f %f\n",
					ratios[2],resons[2],gains[2]);
		printf("ModalParam 0.0 0 3 %f %f %f\n",
					ratios[3],resons[3],gains[3]);
	}
    else    {
        printf("ModalBar : Undefined Control Number!!\n");
    }
}

void ModalBar :: metaControl(int number, char *valString)
{
    float temp[3];
	int which;
	if (number == __SK_Modal_Freq_)	{
		sscanf(valString,"%f %f\n",&temp[0],&temp[1]);
		this->setFreq(temp[0]);
		this->setMasterGain(temp[1]);
	}
    else if (number == __SK_Modal_Input_)	{
		sscanf(valString,"%f %f %f\n",&temp[0],&temp[1],&temp[2]);
		this->setStickHardness(temp[0]);
		this->setStrikePosition(temp[1]);
		this->setDirectGain(temp[2]);
	}
    else if (number == __SK_Modal_Param_)	{
		sscanf(valString,"%i %f %f %f\n",&which,&temp[0],&temp[1],&temp[2]);					
		this->setModalParam(which,temp[0],temp[1],temp[2]);
	}
	else {
		printf("Unknown MetaController Number %i\n",number);
	}
}

