# Tcl/Tk Shakers GUI for the Synthesis Toolkit (STK)

# Set initial control values
set press 64.0
set cont3 64.0
set cont1 64.0
set cont4 64.0
set cont11 64.0
set reverb 10.0
set outID "stdout"
set commtype "stdout"
set patchnum 9

# Configure main window
wm title . "STK Shakers Controller"
wm iconname . "shakers"
. config -bg black

# Configure "communications" menu
menu .menu -tearoff 0
menu .menu.communication -tearoff 0 
.menu add cascade -label "Communication" -menu .menu.communication \
				-underline 0
.menu.communication add radio -label "Console" -variable commtype \
				-value "stdout" -command { setComm }
.menu.communication add radio -label "Socket" -variable commtype \
				-value "socket" -command { setComm }
. configure -menu .menu

# Configure sliders
frame .right -bg black

scale .right.bPressure -from 0 -to 128 -length 300 \
-command {changePress } -variable press\
-orient horizontal -label "Shake Energy" \
-tickinterval 32 -showvalue true -bg grey66

scale .right.cont2 -from 0 -to 128 -length 300 \
-command {printWhatz "ControlChange    0.0 1 " 11} \
-orient horizontal -label "(<--High) System Damping (Low-->)" \
-tickinterval 32 -showvalue true -bg grey66  \
-variable cont11

scale .right.cont4 -from 0 -to 128 -length 300 \
-command {printWhatz "ControlChange    0.0 1 " 4} \
-orient horizontal -label "Number of Objects" \
-tickinterval 32 -showvalue true -bg grey66  \
-variable cont4 

scale .right.cont3 -from 0 -to 128 -length 300 \
-command {printWhatz "ControlChange    0.0 1 " 3} \
-orient horizontal -label "Resonance Resonance" \
-tickinterval 32 -showvalue true -bg grey66  \
-variable cont3 

scale .right.vibrato -from 0 -to 128 -length 300 \
-command {printWhatz "ControlChange    0.0 1 " 1} \
-orient horizontal -label "Resonance Center Freq." \
-tickinterval 32 -showvalue true -bg grey66  \
-variable cont1

scale .right.reverb -from 0 -to 128 -length 300 \
-command {printWhatz "ControlChange    0.0 1 " 44} \
-orient horizontal -label "Reverb" \
-tickinterval 32 -showvalue true -bg grey66  \
-variable reverb  

pack .right.bPressure -padx 10 -pady 10
pack .right.vibrato 	-padx 10 -pady 10
pack .right.cont2 	-padx 10 -pady 10
pack .right.cont3 	-padx 10 -pady 10
pack .right.cont4       -padx 10 -pady 10
pack .right.reverb      -padx 10 -pady 10

pack .right -side right -padx 5 -pady 5

# Configure radio buttons
frame .left -bg black

radiobutton .left.maraca -text Maraca -bg grey66  \
				-command { patchChange 0 } -variable patchnum -width 8 \
				-justify left -value 0
radiobutton .left.sekere -text Sekere -bg grey66 \
				-command { patchChange 2 } -variable patchnum -width 8 \
				-justify left -value 2
radiobutton .left.cabasa -text Cabasa -bg grey66 \
				-command { patchChange 1 } -variable patchnum -width 8 \
				-justify left -value 1
radiobutton .left.bamboo -text Bamboo -bg grey66 \
				-command { patchChange 5 } -variable patchnum -width 8 \
				-justify left -value 5
radiobutton .left.waterdrp -text Waterdrp -bg grey66 \
				-command { patchChange 4 } -variable patchnum -width 8 \
				-justify left -value 4
radiobutton .left.tambourn -text Tambourn -bg grey66 \
				-command { patchChange 6 } -variable patchnum -width 8 \
				-justify left -value 6
radiobutton .left.sleighbl -text Sleighbl -bg grey66 \
				-command { patchChange 7 } -variable patchnum -width 8 \
				-justify left -value 7
radiobutton .left.guiro -text Guiro -bg grey66 \
				-command { patchChange 3 } -variable patchnum -width 8 \
				-justify left -value 3
radiobutton .left.stix1 -text Stix1 -bg grey66 \
				-command { patchChange 8 } -variable patchnum -width 8 \
				-justify left -value 8
radiobutton .left.crunch1 -text Crunch1 -bg grey66 \
				-command { patchChange 9 } -variable patchnum -width 8 \
				-justify left -value 9
radiobutton .left.wrench -text Wrench -bg grey66 \
				-command { patchChange 10 } -variable patchnum -width 8 \
				-justify left -value 10
radiobutton .left.sandpapr -text SandPaper -bg grey66 \
				-command { patchChange 11 } -variable patchnum -width 8 \
				-justify left -value 11
radiobutton .left.cokecan -text CokeCan -bg grey66 \
				-command { patchChange 12 } -variable patchnum -width 8 \
				-justify left -value 12

pack .left.maraca -pady 5
pack .left.sekere -pady 5
pack .left.cabasa -pady 5
pack .left.bamboo -pady 5
pack .left.waterdrp -pady 5
pack .left.tambourn -pady 5
pack .left.sleighbl -pady 5
pack .left.guiro -pady 5
pack .left.stix1 -pady 5
pack .left.crunch1 -pady 5
pack .left.wrench -pady 5
pack .left.sandpapr -pady 5
pack .left.cokecan -pady 5

pack .left -side left -padx 10

# Configure exit and emit buttons
button .left.exit -text "Exit Program" -bg grey66 -command myExit
button .left.poop -text "emit params." -bg grey66 \
	-command {printWhatz "ControlChange    0.0 1 " 1024 0}
pack .left.exit -pady 10
pack .left.poop -pady 2

#bind all <KeyPress> {
bind . <KeyPress> {
    patchChange $patchnum
}

# Bind an X windows "close" event with the Exit routine
bind . <Destroy> +myExit

proc myExit {} {
    global outID
    puts $outID [format "Exit Program\n"]
    flush $outID
    close $outID
    exit
}

proc patchChange {value} {
    global outID press
    puts $outID [format "NoteOn    0.0 1 %i $press" $value]
    flush $outID
}

proc printWhatz {tag value1 value2 } {
		global outID
    puts $outID [format "%s %i %f" $tag $value1 $value2]
    flush $outID
}

proc changePress {value} {
    global outID
    puts $outID [format "AfterTouch       0.0 1 %f" $value]
    flush $outID
}

# Socket connection procedure
  set sockport 2001
  set sockhost localhost
  set err [catch {socket $sockhost $sockport} outID]
